@@ -23,6 +23,7 @@
 
 #include "config.h"
 #include "zenity.h"
+#include <stdlib.h>
 #include <popt.h>
 #include <langinfo.h>
 
@@ -88,6 +89,7 @@ enum {
   OPTION_ERRORTEXT,
   OPTION_INFOTEXT,
   OPTION_FILENAME,
+  OPTION_MULTIFILE,
   OPTION_TEXTFILENAME,
   OPTION_COLUMN,
   OPTION_SEPERATOR,
@@ -97,6 +99,7 @@ enum {
   OPTION_PROGRESSTEXT,
   OPTION_PERCENTAGE,
   OPTION_PULSATE,
+  OPTION_AUTOCLOSE,
   OPTION_QUESTIONTEXT,
   OPTION_WARNINGTEXT,
   OPTION_ABOUT,
@@ -422,6 +425,24 @@ struct poptOption file_selection_options[] = {
     N_("Set the filename"),
     N_("FILENAME")
   },
+  {
+    "multiple",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_MULTIFILE,
+    N_("Allow multiple files to be selected"),
+    NULL
+  },
+  {
+    "separator",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_SEPERATOR,
+    N_("Set output separator character."),
+    N_("SEPARATOR")
+  },
   POPT_TABLEEND
 };
 
@@ -469,7 +490,7 @@ struct poptOption list_options[] = {
     NULL,
     OPTION_SEPERATOR,
     N_("Set output separator character"),
-    NULL
+    N_("SEPARATOR")
   },
   {
     "editable",
@@ -520,6 +541,16 @@ struct poptOption progress_options[] = {
     N_("Pulsate progress bar"),
     NULL
   },
+  {
+    "auto-close",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_AUTOCLOSE,
+    /* xgettext: no-c-format */
+    N_("Dismiss the dialog when 100% has been reached"),
+    NULL
+  },
   POPT_TABLEEND
 };
 
@@ -913,10 +944,13 @@ zenity_init_parsing_options (void) {
   results->calendar_data->month = 0;
   results->calendar_data->year = 0;
   results->calendar_data->dialog_text = NULL;
+  results->file_data->multi = FALSE;
+  results->file_data->separator = g_strdup ("|");
   results->text_data->editable = FALSE;
-  results->tree_data->separator = g_strdup ("/");
+  results->tree_data->separator = g_strdup ("|");
   results->progress_data->percentage = -1;
   results->progress_data->pulsate = FALSE;
+  results->progress_data->autoclose = FALSE;
   results->entry_data->visible = TRUE;
   results->tree_data->checkbox = FALSE;
   results->tree_data->radiobox = FALSE;
@@ -956,6 +990,7 @@ zenity_free_parsing_options (void) {
     case MODE_FILE:
       if (results->file_data->uri)
         g_free (results->file_data->uri);
+	g_free (results->file_data->separator);
       break;
     case MODE_TEXTINFO:
       if (results->text_data->uri)
@@ -974,10 +1009,7 @@ zenity_free_parsing_options (void) {
 
 gint 
 main (gint argc, gchar **argv) {
-  ZenityData *general;
-  ZenityCalendarData *cal_data;
   poptContext ctx;
-  gchar **args;
   gint nextopt, retval;
 
   bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -1000,18 +1032,20 @@ main (gint argc, gchar **argv) {
     /*nothing*/;
 
   if (nextopt != -1) {
-    g_printerr (_("%s in an invalid option for this dialog. See zenity --help for more details\n"),
+    g_printerr (_("%s is an invalid option. See 'zenity --help' for more details\n"),
                   poptBadOption (ctx, 0));
     zenity_free_parsing_options (); 
     exit (-1); 
   }
   gtk_init (&argc, &argv);
 
-  if (argc < 2) {
-    g_printerr (_("You must specify more arguments. See zenity --help for more details\n"));
-    zenity_free_parsing_options ();
-    exit (-1);
-  }
+  /* 
+   * if (argc < 2) {
+   *   g_printerr (_("You must specify more arguments. See zenity --help for more details\n"));
+   *   zenity_free_parsing_options (); 
+   *   exit (-1); 
+   * }
+   */
 
   switch (results->mode) {
     case MODE_CALENDAR:
@@ -1091,7 +1125,7 @@ zenity_parse_options_callback (poptContext              ctx,
   static gint parse_option_text = 0;
   static gint parse_option_file = 0;
   static gint parse_option_editable = 0;
-
+  
   if (reason == POPT_CALLBACK_REASON_POST)
     return;
   else if (reason != POPT_CALLBACK_REASON_OPTION)
@@ -1201,22 +1235,22 @@ zenity_parse_options_callback (poptContext              ctx,
 
       if (parse_option_text > 6) 
         zenity_error ("--text", ERROR_DUPLICATE); 
-      
+     
       switch (results->mode) { 
         case MODE_CALENDAR: 
-          results->calendar_data->dialog_text = g_strdup (arg); 
+          results->calendar_data->dialog_text = g_strdup (g_strcompress (arg)); 
           break; 
         case MODE_ENTRY: 
-          results->entry_data->dialog_text = g_strdup (arg); 
+          results->entry_data->dialog_text = g_strdup (g_strcompress (arg)); 
           break; 
         case MODE_ERROR: 
         case MODE_QUESTION: 
         case MODE_WARNING: 
         case MODE_INFO: 
-          results->msg_data->dialog_text = g_strdup (arg); 
+          results->msg_data->dialog_text = g_strdup (g_strcompress (arg)); 
           break; 
         case MODE_PROGRESS: 
-          results->progress_data->dialog_text = g_strdup (arg); 
+          results->progress_data->dialog_text = g_strdup (g_strcompress (arg)); 
           break; 
         default: 
           zenity_error ("--text", ERROR_SUPPORT); 
@@ -1324,6 +1358,12 @@ zenity_parse_options_callback (poptContext              ctx,
       } 
       parse_option_file++; 
       break; 
+    case OPTION_MULTIFILE:
+      if (results->mode != MODE_FILE)
+        zenity_error ("--multiple", ERROR_SUPPORT);
+
+      results->file_data->multi = TRUE;
+      break;
     case OPTION_COLUMN: 
       if (results->mode != MODE_LIST) 
         zenity_error ("--column", ERROR_SUPPORT); 
@@ -1348,15 +1388,21 @@ zenity_parse_options_callback (poptContext              ctx,
       results->tree_data->radiobox = TRUE; 
       break; 
     case OPTION_SEPERATOR: 
-      if (results->mode != MODE_LIST) 
-        zenity_error ("--separator", ERROR_SUPPORT); 
-      
-      if (parse_option_separator) 
-        zenity_error ("--separator", ERROR_DUPLICATE); 
-      
-      results->tree_data->separator = g_strdup (arg); 
-      parse_option_separator = TRUE; 
-      break; 
+      if (parse_option_separator > 2)
+	zenity_error ("--separator", ERROR_DUPLICATE);
+      switch (results->mode) {
+	case MODE_LIST: 
+      	  results->tree_data->separator = g_strdup (g_strcompress (arg)); 
+      	  parse_option_separator++; 
+	  break;
+	case MODE_FILE:
+	  results->file_data->separator = g_strdup (g_strcompress (arg));
+	  parse_option_separator++;
+	  break;
+	default:
+	  zenity_error ("--separator", ERROR_SUPPORT);
+	}
+      break;
     case OPTION_PERCENTAGE: 
       if (results->mode != MODE_PROGRESS) 
         zenity_error ("--percentage", ERROR_SUPPORT); 
@@ -1372,6 +1418,12 @@ zenity_parse_options_callback (poptContext              ctx,
       
       results->progress_data->pulsate = TRUE; 
       break; 
+    case OPTION_AUTOCLOSE:
+      if (results->mode != MODE_PROGRESS)
+        zenity_error ("--auto-close", ERROR_SUPPORT);
+
+      results->progress_data->autoclose = TRUE;
+      break;
     case OPTION_ABOUT: 
       if (results->mode != MODE_LAST) 
         zenity_error (NULL, ERROR_DIALOG); 
